@@ -4,6 +4,7 @@ import argparse
 import gc
 import sys
 import math
+import time
 from collections import defaultdict
 from decimal import Decimal
 
@@ -94,7 +95,13 @@ def pytest_addoption(parser):
     group.addoption(
         "--benchmark-warmup",
         action="store_true", default=False,
-        help="Runs the benchmarks two times. Discards data from the first run."
+        help="Activates warmup. Will run the test function up to number of times in the calibration phase. "
+             "See `--benchmark-warmup-iterations`."
+    )
+    group.addoption(
+        "--benchmark-warmup-iterations",
+        action="store", type=int, default=100000,
+        help="Max number of iterations to run in the warmup phase. Default: %(default)s"
     )
     group.addoption(
         "--benchmark-verbose",
@@ -142,7 +149,8 @@ class BenchmarkFixture(object):
         else:
             return cls._precisions.setdefault(timer, compute_timer_precision(timer))
 
-    def __init__(self, name, disable_gc, timer, min_rounds, min_time, max_time, warmup, add_stats, logger, group=None):
+    def __init__(self, name, disable_gc, timer, min_rounds, min_time, max_time, warmup, warmup_iterations,
+                 add_stats, logger, group=None):
         self._disable_gc = disable_gc
         self._name = name
         self._group = group
@@ -151,24 +159,23 @@ class BenchmarkFixture(object):
         self._max_time = float(max_time)
         self._min_time = float(min_time)
         self._add_stats = add_stats
-        self._warmup = warmup
+        self._warmup = warmup and warmup_iterations
         self._logger = logger
 
     def __call__(self, function_to_benchmark):
-        def runner(loops, timer=self._timer):
-            loops_range = XRANGE(loops)
+        def runner(loops_range, timer=self._timer):
             gcenabled = gc.isenabled()
             if self._disable_gc:
                 gc.disable()
             start = timer()
-            for index in loops_range:
+            for _ in loops_range:
                 function_to_benchmark()
             end = timer()
             if gcenabled:
                 gc.enable()
             return end - start
 
-        duration, scale = self._calibrate_timer(runner)
+        duration, scale, loops_range = self._calibrate_timer(runner)
 
         # Choose how many time we must repeat the test
         rounds = int(math.ceil(self._max_time / duration))
@@ -177,12 +184,12 @@ class BenchmarkFixture(object):
         stats = BenchmarkStats(self._name, group=self._group, scale=scale)
         self._add_stats(stats)
 
-        if self._warmup:
-            for _ in XRANGE(rounds):
-                runner(scale)
-
+        self._logger.write("")
+        self._logger.write("  Running %s rounds ..." % rounds)
+        run_start = time.time()
         for _ in XRANGE(rounds):
-            stats.update(runner(scale))
+            stats.update(runner(loops_range))
+        self._logger.write("  Ran for %ss." % time_format(time.time() - run_start), bold=True)
 
         return function_to_benchmark()
 
@@ -197,8 +204,19 @@ class BenchmarkFixture(object):
 
         loops = 1
         while True:
-            duration = runner(loops)
-            self._logger.write("  Calibrate: %ss for %s iterations." % (time_format(duration), loops))
+            loops_range = XRANGE(loops)
+            duration = runner(loops_range)
+            if self._warmup:
+                warmup_start = time.time()
+                warmup_iterations = 0
+                while time.time() - warmup_start < self._max_time and warmup_iterations < self._warmup:
+                    duration = min(duration, runner(loops_range))
+                    warmup_iterations += 1
+                self._logger.write("    warmed up for %ss (%s iterations)." % (
+                    time_format(time.time() - warmup_start),
+                    warmup_iterations
+                ))
+            self._logger.write("  - measured %s iterations: %ss." % (loops, time_format(duration)))
 
             if duration / min_time >= 0.75:
                 break
@@ -206,10 +224,10 @@ class BenchmarkFixture(object):
             if duration >= min_time_estimate:
                 # coarse estimation of the number of loops
                 loops = int(min_time * loops / duration)
-                self._logger.write("  Calibrate estimate: %s iterations." % loops)
+                self._logger.write("  * estimating %s iterations." % loops)
             else:
                 loops *= 10
-        return duration, loops
+        return duration, loops, loops_range
 
 
 def time_unit(value):
@@ -244,11 +262,11 @@ class DiagnosticLogger(object):
         self.term = verbose and py.io.TerminalWriter()
         # self.capman = capman
 
-    def write(self, text):
+    def write(self, text, **kwargs):
         if self.term:
             # if self.capman:
             #     self.capman.suspendcapture(in_=True)
-            self.term.line(text, yellow=True)
+            self.term.line(text, yellow=True, **kwargs)
             # if self.capman:
             #     self.capman.resumecapture()
 
@@ -262,7 +280,8 @@ class BenchmarkSession(object):
             max_time=config.getoption("benchmark_max_time"),
             timer=timer,
             disable_gc=config.getoption("benchmark_disable_gc"),
-            warmup=config.getoption("benchmark_warmup")
+            warmup=config.getoption("benchmark_warmup"),
+            warmup_iterations=config.getoption("benchmark_warmup_iterations"),
         )
         self._skip = config.getoption("benchmark_skip")
         self._only = config.getoption("benchmark_only")
